@@ -37,7 +37,7 @@ async def _invoke(callback: Callable, *params: object) -> Any:
     return result
 
 
-async def invoke(callback: Callable, *params: object) -> Any:
+async def invoke(callback: Callable[[], Any], *params: object) -> Any:
     """Invoke a callback with an arbitrary number of parameters.
 
     Args:
