@@ -1,10 +1,11 @@
 from __future__ import annotations
 
 import hashlib
+import inspect
 import os
 import shlex
 from pathlib import Path
-from typing import Iterable, cast
+from typing import Awaitable, Callable, Iterable, cast
 
 from textual._import_app import import_app
 from textual.app import App
@@ -32,8 +33,15 @@ def format_svg(source, language, css_class, options, md, attrs, **kwargs) -> str
         try:
             rows = int(attrs.get("lines", 24))
             columns = int(attrs.get("columns", 80))
+            hover = attrs.get("hover", "")
             svg = take_svg_screenshot(
-                None, path, press, title, terminal_size=(columns, rows)
+                None,
+                path,
+                press,
+                hover=hover,
+                title=title,
+                terminal_size=(columns, rows),
+                wait_for_animation=False,
             )
         finally:
             os.chdir(cwd)
@@ -52,8 +60,11 @@ def take_svg_screenshot(
     app: App | None = None,
     app_path: str | None = None,
     press: Iterable[str] = (),
+    hover: str = "",
     title: str | None = None,
     terminal_size: tuple[int, int] = (80, 24),
+    run_before: Callable[[Pilot], Awaitable[None] | None] | None = None,
+    wait_for_animation: bool = True,
 ) -> str:
     """
 
@@ -61,13 +72,17 @@ def take_svg_screenshot(
         app: An app instance. Must be supplied if app_path is not.
         app_path: A path to an app. Must be supplied if app is not.
         press: Key presses to run before taking screenshot. "_" is a short pause.
+        hover: Hover over the given widget.
         title: The terminal title in the output image.
         terminal_size: A pair of integers (rows, columns), representing terminal size.
+        run_before: An arbitrary callable that runs arbitrary code before taking the
+            screenshot. Use this to simulate complex user interactions with the app
+            that cannot be simulated by key presses.
+        wait_for_animation: Wait for animation to complete before taking screenshot.
 
     Returns:
         An SVG string, showing the content of the terminal window at the time
             the screenshot was taken.
-
     """
 
     if app is None:
@@ -86,11 +101,11 @@ def take_svg_screenshot(
             assert path is not None
             with open(path, "rb") as source_file:
                 hash.update(source_file.read())
-        hash.update(f"{press}-{title}-{terminal_size}".encode("utf-8"))
+        hash.update(f"{press}-{hover}-{title}-{terminal_size}".encode("utf-8"))
         cache_key = f"{hash.hexdigest()}.svg"
         return cache_key
 
-    if app_path is not None:
+    if app_path is not None and run_before is None:
         screenshot_cache = Path(SCREENSHOT_CACHE)
         screenshot_cache.mkdir(exist_ok=True)
 
@@ -100,9 +115,18 @@ def take_svg_screenshot(
 
     async def auto_pilot(pilot: Pilot) -> None:
         app = pilot.app
-        await pilot.press(*press)
-        await pilot.wait_for_scheduled_animations()
+        if run_before is not None:
+            result = run_before(pilot)
+            if inspect.isawaitable(result):
+                await result
         await pilot.pause()
+        await pilot.press(*press)
+        if hover:
+            await pilot.hover(hover)
+            await pilot.pause(0.5)
+        if wait_for_animation:
+            await pilot.wait_for_scheduled_animations()
+            await pilot.pause()
         svg = app.export_screenshot(title=title)
 
         app.exit(svg)
@@ -116,7 +140,7 @@ def take_svg_screenshot(
         ),
     )
 
-    if app_path is not None:
+    if app_path is not None and run_before is None:
         screenshot_path.write_text(svg)
 
     assert svg is not None
