@@ -1,8 +1,8 @@
 from __future__ import annotations
 
 import os
-from pathlib import PurePath
-from typing import Callable
+from pathlib import Path
+from typing import Callable, Iterable, Sequence
 
 import rich.repr
 
@@ -13,14 +13,25 @@ from ._callback import invoke
 class FileMonitor:
     """Monitors files for changes and invokes a callback when it does."""
 
-    def __init__(self, paths: list[PurePath], callback: Callable) -> None:
-        self.paths = paths
+    _paths: set[Path]
+
+    def __init__(self, paths: Sequence[Path], callback: Callable[[], None]) -> None:
+        """Monitor the given file paths for changes.
+
+        Args:
+            paths: Paths to monitor.
+            callback: Callback to invoke if any of the paths change.
+        """
+        self._paths = set(paths)
         self.callback = callback
         self._modified = self._get_last_modified_time()
 
+    def __rich_repr__(self) -> rich.repr.Result:
+        yield self._paths
+
     def _get_last_modified_time(self) -> float:
         """Get the most recent modified time out of all files being watched."""
-        return max(os.stat(path).st_mtime for path in self.paths)
+        return max((os.stat(path).st_mtime for path in self._paths), default=0)
 
     def check(self) -> bool:
         """Check the monitored files. Return True if any were changed since the last modification time."""
@@ -29,6 +40,14 @@ class FileMonitor:
         self._modified = modified
         return changed
 
+    def add_paths(self, paths: Iterable[Path]) -> None:
+        """Adds paths to start being monitored.
+
+        Args:
+            paths: The paths to be monitored.
+        """
+        self._paths.update(paths)
+
     async def __call__(self) -> None:
         if self.check():
             await self.on_change()
