@@ -115,7 +115,7 @@ def test_get_opacity_default():
 
 def test_styles_css_property():
     css = "opacity: 50%; text-opacity: 20%; background: green; color: red; tint: dodgerblue 20%;"
-    styles = Styles().parse(css, path="")
+    styles = Styles().parse(css, read_from=("", ""))
     assert styles.css == (
         "background: #008000;\n"
         "color: #FF0000;\n"
@@ -148,6 +148,13 @@ def test_opacity_set_invalid_type_error():
         styles.text_opacity = "invalid value"
 
 
+def test_opacity_set_allows_integer_value():
+    """Regression test for https://github.com/Textualize/textual/issues/3414"""
+    styles = RenderStyles(DOMNode(), Styles(), Styles())
+    styles.text_opacity = 0
+    assert styles.text_opacity == 0.0
+
+
 @pytest.mark.parametrize(
     "size_dimension_input,size_dimension_expected_output",
     [
