@@ -1,5 +1,4 @@
 from contextlib import nullcontext as does_not_raise
-from typing import Any
 
 import pytest
 
@@ -13,7 +12,7 @@ from textual.widget import Widget
 
 def _make_user_stylesheet(css: str) -> Stylesheet:
     stylesheet = Stylesheet()
-    stylesheet.source["test.css"] = CssSource(css, is_defaults=False)
+    stylesheet.source["test.tcss"] = CssSource(css, is_defaults=False)
     stylesheet.parse()
     return stylesheet
 
