@@ -39,7 +39,7 @@ class NoBindings(App[None]):
 async def test_just_app_no_bindings() -> None:
     """An app with no bindings should have no bindings, other than ctrl+c."""
     async with NoBindings().run_test() as pilot:
-        assert list(pilot.app._bindings.keys.keys()) == ["ctrl+c"]
+        assert list(pilot.app._bindings.keys.keys()) == ["ctrl+c", "ctrl+backslash"]
         assert pilot.app._bindings.get_key("ctrl+c").priority is True
 
 
@@ -60,7 +60,9 @@ class AlphaBinding(App[None]):
 async def test_just_app_alpha_binding() -> None:
     """An app with a single binding should have just the one binding."""
     async with AlphaBinding().run_test() as pilot:
-        assert sorted(pilot.app._bindings.keys.keys()) == sorted(["ctrl+c", "a"])
+        assert sorted(pilot.app._bindings.keys.keys()) == sorted(
+            ["ctrl+c", "ctrl+backslash", "a"]
+        )
         assert pilot.app._bindings.get_key("ctrl+c").priority is True
         assert pilot.app._bindings.get_key("a").priority is True
 
@@ -82,7 +84,9 @@ class LowAlphaBinding(App[None]):
 async def test_just_app_low_priority_alpha_binding() -> None:
     """An app with a single low-priority binding should have just the one binding."""
     async with LowAlphaBinding().run_test() as pilot:
-        assert sorted(pilot.app._bindings.keys.keys()) == sorted(["ctrl+c", "a"])
+        assert sorted(pilot.app._bindings.keys.keys()) == sorted(
+            ["ctrl+c", "ctrl+backslash", "a"]
+        )
         assert pilot.app._bindings.get_key("ctrl+c").priority is True
         assert pilot.app._bindings.get_key("a").priority is False
 
